package ast;

public class Binary extends Expression {
    private enum BinaryOperator {
        ADD("+"), SUB("-"), MUL("*"), DIV("/"), PWR("^");
        private String text;
        private BinaryOperator(String text) {
            this.text = text;
        }
        public static BinaryOperator getBinaryOperator(String text) {
            for (BinaryOperator b : BinaryOperator.values()) {
                if (b.toString().equals(text)) {
                    return b;
                }
            }
            return null;
        }
        public String toString() {
            return this.text;
        }
    }

    private BinaryOperator op = null;
    private Expression lhsNode = null;
    private Expression rhsNode = null;
    public Binary(String op, Expression lhs, Expression rhs) {
        this.op = BinaryOperator.getBinaryOperator(op);
        this.lhsNode = lhs;
        this.rhsNode = rhs;
    }
    public double evaluate(Program p) {
        double lhs = this.lhsNode.evaluate(p);
        double rhs = this.rhsNode.evaluate(p);
        switch (this.op) {
            case ADD: return lhs + rhs;
            case SUB: return lhs - rhs;
            case MUL: return lhs * rhs;
            case DIV: return lhs / rhs;
            case PWR: return Math.pow(lhs, rhs);
        }
        return 0.0;
    }
    public String toString() {
        return lhsNode.toString() + " " + op.toString() + " " + rhsNode.toString();
    }
}
