package ast;

import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;

public class MultiArg extends Expression {
    private enum MultiArgOperator {
        MIN("min"), MAX("max");
        private String text;
        private MultiArgOperator(String text) {
            this.text = text;
        }
        public static MultiArgOperator getMultiArgOperator(String text) {
            for (MultiArgOperator b : MultiArgOperator.values()) {
                if (b.toString().equals(text)) {
                    return b;
                }
            }
            return null;
        }
        public String toString() {
            return this.text;
        }
    }
    private MultiArgOperator op = null;
    private List<Expression> arguments;
    public MultiArg(String op, Expression first) {
        this.op = MultiArgOperator.getMultiArgOperator(op);
        arguments = new ArrayList<Expression>();
        arguments.add(first);
    }
    public void addArgument(Expression arg) {
        arguments.add(arg);
    }
    public double evaluate(Program prog) {
        Iterator<Expression> args = arguments.iterator();
        double retval = args.next().evaluate(prog);
        while (args.hasNext()) {
            switch (this.op) {
                case MIN:
                    retval = Math.min(retval, args.next().evaluate(prog));
                    break;
                case MAX:
                    retval = Math.max(retval, args.next().evaluate(prog));
                    break;
            }
        }
        return retval;
    }
    public String toString() {
        StringBuffer str = new StringBuffer("(");
        Iterator<Expression> args = arguments.iterator();
        str.append(args.next());
        while (args.hasNext()) {
            str.append(", ");
            str.append(args.next());
        }
        str.append(")");
        return str.toString();
    }
}
