package ast;

public class Unary extends Expression {
    private enum UnaryOperator {
        PLS("+"), NEG("-"), ABS("abs");
        private String text;
        private UnaryOperator(String text) {
            this.text = text;
        }
        public static UnaryOperator getUnaryOperator(String text) {
            for (UnaryOperator b : UnaryOperator.values()) {
                if (b.toString().equals(text)) {
                    return b;
                }
            }
            return null;
        }
        public String toString() {
            return this.text;
        }
    }

    private UnaryOperator op = null;
    private Expression node;
    public Unary(String op, Expression e) {
        this.op = UnaryOperator.getUnaryOperator(op);
        this.node = e;
    }
    public double evaluate(Program p) {
        double val = this.node.evaluate(p);
        switch (this.op) {
            case PLS: return val;
            case NEG: return -val;
            case ABS: return (val < 0) ? -val : val;
        }
        return 0.0;
    }
    public String toString() {
	if (this.op == UnaryOperator.ABS) {
            return this.op.toString() + "(" + this.node.toString() + ")";
        } else {
            return this.op.toString() + this.node.toString();
        }
    }
}
